/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;


/**
 * This class extends the basic coupled random field by using independent
 * Gaussian proposals for the couplings.
 * @author L.A. Breyer
 * @version 1.0
 */
class CoupledGaussianField extends CouplableField {

    final static int NUMCOLLAPSED = 50;
    
    double msigma2, mmu, eps;

    double proposal, propMu, propSigma, propSigma2, E;
    double xi, z0;
    RealInterval sigmaRange, muRange;
    
    int numbins, z0bin;
    boolean z0binCollapsed = false;
    double collapseds[][];
    int numcollapsed[];

    RandomField original = null;
    CoupledGaussianField previous = null;
    GaussianEstimates gest;

    CoupledGaussianField() { // dummy
    }

    CoupledGaussianField(int numb, RandomField field, GaussianEstimates g) {
	original = field;
	numbins = numb;
	gest = g;

	collapseds = new double[numbins][NUMCOLLAPSED];
	numcollapsed = new int[numbins];
    }

    CoupledGaussianField respawn2(int numb, RandomField field, GaussianEstimates g) {
	original = field;
	previous = null;
	z0binCollapsed = false;
	numbins = numb;
	gest = g;

	//collapseds = new double[numbins][NUMCOLLAPSED];
	//numcollapsed = new int[numbins];

	if( collapseds != null ) {
	    if( collapseds.length == numb ) {
		for(int i = 0; i < numb; i++) {
		    numcollapsed[i] = 0;
		    for(int j = 0; j < NUMCOLLAPSED; j++) {
			collapseds[i][j] = 0.0;
		    }
		}
	    } else {
		collapseds = new double[numbins][NUMCOLLAPSED];
		numcollapsed = new int[numbins];
	    }
	} else {
	    collapseds = new double[numbins][NUMCOLLAPSED];
	    numcollapsed = new int[numbins];
	}
	

	msigma2 = mmu = eps = 0.0;

	proposal = propMu = propSigma = propSigma2 = E = 0.0;
	xi = z0 = 0.0;
	sigmaRange = muRange = null;
    
        z0bin = 0;

	return this;
    }
    
    CoupledGaussianField(MyRandom r, int numb, CoupledGaussianField field,
			       GaussianEstimates g) {
	previous = field;
	original = null;
	numbins = numb;
	eps = (double)1/( 2 * numbins);
	gest = g;

	if( previous != null ) {
	    collapseds = previous.getCollapseds();
	    numcollapsed = previous.getNumCollapsed();
	} 
	
	xi = r.nextDouble();
	z0bin = (int)Math.floor(r.nextDouble() * (double)numbins);
	z0 = (double)z0bin * 2 * eps + eps;

	RealInterval bin = new RealInterval(z0 - eps, z0 + eps); // note this must belong to [0,1]
	sigmaRange = gest.sigmaRange(bin);
	muRange = gest.muRange(bin);
	
	propMu = muRange.midpoint(); //fz0 = gest.muAt(z0); 
	propSigma = sigmaRange.end;
	propSigma2 = propSigma * propSigma;
	E = muRange.length()/2;
	
	proposal = propMu + r.nextPlateauGaussian(propSigma, E); 

//	double sigma = gest.maxSigma(z0, eps);
//	msigma2 = sigma * sigma;
//	mmu = gest.maxMu(z0, eps);
	
	double Y = Math.max(Math.abs(proposal- propMu), E);
	
	if( Math.exp( - 2 * E * Y / propSigma2) > xi )
	    z0binCollapsed = true;
    }

    CoupledGaussianField respawn(MyRandom r, int numb, CoupledGaussianField field,
			       GaussianEstimates g) {
	previous = field;
	original = null;
	numbins = numb;
	eps = (double)1/( 2 * numbins);
	gest = g;

	z0binCollapsed = false;
	collapseds = null;
	numcollapsed = null;
//  	if( previous != null ) {
//  	    collapseds = previous.getCollapseds();
//  	    numcollapsed = previous.getNumCollapsed();
//  	} else {
//  	    if( collapseds.length == numb ) {
//  		for(int i = 0; i < numb; i++) {
//  		    numcollapsed[i] = 0;
//  		    for(int j = 0; j < NUMCOLLAPSED; j++) {
//  			collapseds[i][j] = 0.0;
//  		    }
//  		}
//  	    } else {
//  		collapseds = new double[numbins][NUMCOLLAPSED];
//  		numcollapsed = new int[numbins];
//  	    }
//  	}

	xi = r.nextDouble();
	z0bin = (int)Math.floor(r.nextDouble() * (double)numbins);
	z0 = (double)z0bin * 2 * eps + eps;

	RealInterval bin = new RealInterval(z0 - eps, z0 + eps); // note this must belong to [0,1]
	sigmaRange = gest.sigmaRange(bin);
	muRange = gest.muRange(bin);
	
	propMu = muRange.midpoint(); //fz0 = gest.muAt(z0); 
	propSigma = sigmaRange.end;
	propSigma2 = propSigma * propSigma;
	E = muRange.length()/2;
	
	proposal = propMu + r.nextPlateauGaussian(propSigma, E); 

//	double sigma = gest.maxSigma(z0, eps);
//	msigma2 = sigma * sigma;
//	mmu = gest.maxMu(z0, eps);
	
	double Y = Math.max(Math.abs(proposal- propMu), E);
	
	if( Math.exp( - 2 * E * Y / propSigma2) > xi )
	    z0binCollapsed = true;

	return this;
    }



    RealInterval getMuRange() {
	return muRange;
    }
    
    
    double[][] getCollapseds() {
	return collapseds;
    }

    int[] getNumCollapsed() {
	return numcollapsed;
    }
    
    public boolean hasCollapsed(int binno) {
	if(original != null) 
	    return false;
	else if(previous == null) {  // this is for preliminary test
	    if(binno == z0bin)
		return aPrioriCollapse(binno);
	    else
		return false;
	} else 
	    // return ((numcollapsed[binno] == 1) && (binno == z0bin));
	    return ((binno == z0bin) && (z0binCollapsed));
    }

    public boolean aPrioriCollapse(int binno) {
	if( (binno != z0bin) || (original != null) )
	    return false;
	else
	    return z0binCollapsed;
    }

    void queryCollapse() throws ArrayIndexOutOfBoundsException {
	    // compute proposal and check if z0bin has collapsed
	if( z0binCollapsed ) { //aPrioriCollapse(z0bin) ) {
	    collapseds[z0bin][0] = proposal; // add to list
	    numcollapsed[z0bin] = 1;
	}	

	    // now update all bins with the proposal
	for(int k = 0; k < numbins; k++)
	    if( numcollapsed[k] > 0 ) {
  		RealInterval r = new RealInterval(k * eps, (k+1) * eps);				        

		int keepIt = 0, chuckIt = 0, index = 0;
		boolean gotIt = false;
		for( int l = 0; l < numcollapsed[k]; l++ ) { // go through range of bin
		    double X = collapseds[k][l];

			// does everyone in the bin who is at X refuse to jump?
		    if( proposalDensityAt(proposal) * xi >=
			gest.bEstimate(r, X, proposal) * proposalDensityAt(X) ) {
			chuckIt++;
			collapseds[k][index++] = X; // save X value & increment index			
		    } else {
			    // some may have accepted
			    // does everyone in the bin who is at X want to jump?
			if( proposalDensityAt(proposal) * xi < 
			    gest.bTildeEstimate(r, X, proposal) * proposalDensityAt(X) ) {
			    if( !gotIt )
				collapseds[k][index++] = proposal; // replace the value X & increment index
			    chuckIt++; // no need to keep it anymore
			    gotIt = true;
			} else { // some may have refused
			    keepIt++;
			    collapseds[k][index++] = X; // so must keep existing X value
			}
		    }
		}

		if( !gotIt && (keepIt > 0) ) { // this will throw an exception
		    collapseds[k][index++] = proposal; // if numcollapsed[k] exceeds
			// the value NUMCOLLAPSED
		}
		numcollapsed[k] = index; // update the number of values stored by bin
	    }	     
    }

      
    public CoupledGaussianField applyTo(CoupledGaussianField field) {
	if( (field != null) && (original == null) ) {
	    previous = field;
	    collapseds = previous.getCollapseds();
	    numcollapsed = previous.getNumCollapsed();
	    queryCollapse(); // because we lost information previously stored in bins 
	}
	return this;
    }

    public int numBins() {
	return numbins;
    }

    double proposalDensityAt(double x) {
	if( x <= propMu-E ) {
	    double t = x - propMu + E;
	    return Math.exp( - t * t / (2 * propSigma2));
	}
	else if( x <= propMu+E)
	    return 1;
	else {
	    double t = x - propMu - E;
	    return Math.exp( - t * t / (2 * propSigma2));
	}
    }
    
    public double valueAt(double x) {
	if( (x < 0) || (x > 1) )
	    return 0;
	
	if( previous != null ) {
	    int binno = (int)Math.floor( x / (2 * eps) );
	    if( (binno == z0bin) && z0binCollapsed ) 
		return proposal; 
	    else {
		double fv = previous.valueAt(x);
		if( gest.kernelDensity(x, proposal) * proposalDensityAt(fv)
		    > xi * gest.kernelDensity(x, fv) * proposalDensityAt(proposal) )
		    return proposal;
		else 
		    return fv;
	    }
	} else if( original != null ) {
	    return original.valueAt(x);
	} else {
	    return 0.0; //original.valueAt(x); // this sets off an exception
	}
    }

    public double kernelDensity(double x, double y) {
	return gest.kernelDensity(x,y);
    }

    public double getz0() {
	return z0;
    }

    public double getProposal() {
	return proposal;
    }
    

//     public double kernelMean(double x) {
// 	if( oldField == null )
// 	    return 0;
// 	else
// 	    return oldField.kernelMean(x);
//     }

//     public double kernelVariance(double x) {
// 	if( oldField == null )
// 	    return 0;
// 	else
// 	    return oldField.kernelVariance(x);
//     }

//     public double newVariableAt(Random r, double x) {
// 	if( oldField == null )
// 	    return 0;
// 	else
// 	    return oldField.newVariableAt(r, x);
//     }

}








