/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class implements a random field suitable for using
 * as a proposal with the Random walk Metropolis sampler, using a Gaussian
 * proposal distribution.
 * @author L.A. Breyer
 * @version 1.0
 */
final class GaussianRandomWalkField extends CouplableField 
implements GaussianEstimates {

    double w;
    double sigma, sigma2, sigma22, oo2sigma2;

	/**
	 * This constructor accepts a value of
	 * s, the magnification factor for the increment.
	 * @param r the random number generator
	 * @param s the standard deviation for the increment
	 */
    GaussianRandomWalkField(Random r, double s) {
	w = r.nextGaussian();
	sigma = s;
	sigma2 = s * s;
	oo2sigma2 = 1 / (2 * sigma2);
	w = sigma * w;
    }

    GaussianRandomWalkField respawn(Random r, double s) {
	w = r.nextGaussian();
	sigma = s;
	sigma2 = s * s;
	oo2sigma2 = 1 / (2 * sigma2);
	w = sigma * w;
	return this;
    }

    public double valueAt(double x) {
	return x + w;
    }

    public double sigmaAt(double z) {
	return sigma;
    }

    public double muAt(double z) {
	return z;
    }

    public RealInterval sigmaRange(RealInterval r) {
	return new RealInterval(sigma, sigma);
    }

    public RealInterval muRange(RealInterval r) {
	return r;
    }
    
	/**
	 * note: estimate assumes that r is a subinterval of [0,1]
	 */
    public double bEstimate(RealInterval r, double x, double y) {
	double t = y * y - x * x;
	if( y > x )
	    t -= 2 * r.end * (y - x);
	else
	    t -= 2 * r.beginning * (y - x);

	return Math.exp( - t / (2 * sigma2) );
    }

	/**
	 * note: estimate assumes that r is a subinterval of [0,1]. 
	 */
    public double bTildeEstimate(RealInterval r, double x, double y) {
	double t = y * y - x * x;
	if( y > x )
	    t -= 2 * r.beginning * (y - x);
	else
	    t -= 2 * r.end * (y - x);

	return Math.exp( - t / (2 * sigma2) );
    }
    
    public double newVariableAt(Random r, double x) {
	double w = r.nextGaussian();
	return x + sigma * w;
    }

    public double kernelDensity(double x, double y) {
	double t = (y-x);
	return Math.exp( - t * t * oo2sigma2 );	
    }

    public double maxSigma(double z0, double eps) {
	return sigma;
    }

    public double maxMu(double z0, double eps) {
	return eps;
    }

    public double centerMu(double z0) {
	return z0;
    }    

//    public double kernelMean(double x) {
//	return x;
//   }

//    public double kernelVariance(double x) {
//	return sigma2;
//    }

}
