/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.util.Random;

/**
 * This class implements a hot pdf, that is a pdf which is raised to some
 * inverse temperature beta.
 *
 * @author L.A. Breyer
 * @version 1.0
 */
final class HotPdf extends Pdf {

    double beta;
    Pdf pdf;

	/**
	 * This constructor creates a hot pdf, by taking an existing pdf and raising it to
	 * the inverse temperature beta.
	 * @param a
	 * @param b
	 */
    HotPdf(double itemp, Pdf aPdf) {
	pdf = aPdf;
	beta = itemp;
    }

	// dummy
    public double newVariable(Random r) {
	return -1;
    }
 
    public double valueAt(double x) {
	return Math.pow(pdf.valueAt(x), beta);
    }

//    public double densityRatio(double x, double y) {
//	if( (x < 0) || (x > 1) )
//	    return 0;
//	else
//	    return Math.exp( lambda *(x - y));
//   }
    
    public RealInterval getDensityRange() {
	RealInterval i = pdf.getDensityRange();
	if( beta >= 0 )
	    return new RealInterval( Math.pow(i.beginning, beta), Math.pow(i.end, beta) );
	else
	    return new RealInterval( Math.pow(i.end, beta), Math.pow(i.beginning, beta) );	    
    }

    public double gradLogAt(double x) {
	return beta * pdf.gradLogAt(x);
    }

    public RealInterval gradLogRange(RealInterval r) {
	RealInterval i = pdf.gradLogRange(r);
	return new RealInterval( beta * r.beginning, beta * r.end);
    }

    public RealInterval range(RealInterval xrange) {
	RealInterval i = pdf.range(xrange);
	if( beta >= 0 )
	    return new RealInterval( Math.pow(i.beginning, beta), Math.pow(i.end, beta) );
	else
	    return new RealInterval( Math.pow(i.end, beta), Math.pow(i.beginning, beta) );	
    }

	// dummy
    public double getMean() {
	return 0;
   }

	// dummy
    public double getVariance() {
	return 1;
    }
}



