/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class implements a random field suitable for use
 * as a proposal with the Langevin sampler. 
 * @author L.A. Breyer
 * @version 1.0
 */
final class LangevinField extends CouplableField
implements GaussianEstimates {

    double w, sigma, sigma2, sigma22;
    Pdf target;

	/**
	 * This constructor accepts a target density for the drift,
	 * and a magnification factor s for the Gaussian increment.
	 * Note that the Gaussian increment is _truncated_ to within
	 * 3 standard deviations.
	 * @param r the random number generator
	 * @param targetDensity a Pdf used to compute the drift
	 * @param s the standard deviation for the Gaussian increment
	 */
    LangevinField(Random r, Pdf targetDensity, double s) {
	w = r.nextGaussian();
	w = s * w;
	sigma = s;
	target = targetDensity;
	sigma2 = s * s;
	sigma22 = sigma2 / 2;
    }

    LangevinField respawn(Random r, Pdf targetDensity, double s) {
	w = r.nextGaussian();
	w = s * w;
	sigma = s;
	target = targetDensity;
	sigma2 = s * s;
	sigma22 = sigma2 / 2;
	return this;
    }
    
    public double valueAt(double x) {
	return x + w + target.gradLogAt(x) * sigma22;
    }

    public double sigmaAt(double z) {
	return sigma;
    }

    public double muAt(double z) {
	return z + target.gradLogAt(z) * sigma22;
    }

    public RealInterval sigmaRange(RealInterval r) {
	return new RealInterval(sigma, sigma);
    }

    public RealInterval muRange(RealInterval r) {
	RealInterval tm = target.gradLogRange(r);
	tm.beginning = sigma22 * tm.beginning + r.beginning; 
	tm.end = sigma22 * tm.end + r.end;
	return tm;
    }

    public double bEstimate(RealInterval r, double x, double y) {
	    // compute interval containing the mean
	RealInterval tm = target.gradLogRange(r);
	tm.end = tm.end * sigma22 + r.end;
	tm.beginning = tm.beginning * sigma22 + r.beginning;

	double t = y * y - x * x;
	if( y > x )
	    t -= 2 * tm.end * (y - x);
	else
	    t -= 2 * tm.beginning * (y - x);

	return Math.exp( - t / (2 * sigma2) );
    }

    public double bTildeEstimate(RealInterval r, double x, double y) {
	    // compute interval containing the mean
	RealInterval tm = target.gradLogRange(r);
	tm.end =  tm.end * sigma22 + r.end;
	tm.beginning = tm.beginning * sigma22 + r.beginning;

	double t = y * y - x * x;
	if( y > x )
	    t -= 2 * tm.beginning * (y - x);
	else
	    t -= 2 * tm.end * (y - x);

	return Math.exp( - t / (2 * sigma2) );
    }
    
    public double newVariableAt(Random r, double x) {
	double w = r.nextGaussian();
	w = sigma * w;
	return x + w + target.gradLogAt(x) * sigma22;
    }

    public double kernelDensity(double x, double y) {
	double t = y - x - sigma22 * target.gradLogAt(x);
	if( t * t > 9 * sigma * sigma )
	    return 0;
	else
	    return Math.exp( - t * t / (4 * sigma22) );
    }
 
    public double maxSigma(double z0, double eps) {
	return sigma;
    }

    public double maxMu(double z0, double eps) {
	RealInterval tm = target.gradLogRange(new RealInterval(z0 - eps, z0 + eps));
	double glz0 = target.gradLogAt(z0);
	return eps + sigma22 * Math.max( Math.abs(tm.beginning - glz0), Math.abs(tm.end - glz0) );
    }

    public double centerMu(double z0) {
	return z0 + target.gradLogAt(z0) * sigma22;
    }
   
   public double kernelMean(double x) {
	return x + target.gradLogAt(x) * sigma22;
    }

    public double kernelVariance(double x) {
	return 2 * sigma22;
    }
}
