/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.util.Random;

/**
 * This class implements an (unnormalized) Gaussian probability density,
 * truncated so as to live on the interval (0,1).
 * @author L.A. Breyer
 * @version 1.0
 */
final class Normal extends Pdf {

    double mu, sigma2, logmax;

	/**
	 * This constructor creates the density with mean m and variance v.
	 * @param m the mean
	 * @param v the variance
	 */
    Normal(double m, double v) {
	mu = m;
	sigma2 = v;
	if( (mu < 0) || (mu > 1))
	    logmax =  - mu * mu / (2*sigma2);
	else 
	    logmax = 0;
    }

	/**
	 * This method returns a Normal (mean m, variance v) random
	 * variable. Note that this variable is *not* truncated to (0,1).
	 * @param r a random number generator
	 * @return a Gaussian random variable
	 */
    public double newVariable(Random r) {
	return r.nextGaussian() * Math.sqrt(sigma2) + mu;
    }
    
    public double valueAt(double x) {
	if( (x < 0) || (x > 1) )
	    return 0;
	else {
	    double t = x - mu;
	    return Math.exp( - t*t/(2*sigma2) - logmax );
	}
    }

    public double densityRatio(double x, double y) {
	if( (x < 0) || (x > 1) )
	    return 0;
	else
	    return Math.exp(((x - mu)*(x - mu) - (y - mu)*(y - mu))/
			    2*sigma2);
    }
    
    public RealInterval getDensityRange() {
	return new RealInterval(0,Math.exp( logmax ));
    }

    public double gradLogAt(double x) {
	if( (x < 0) || (x > 1) )
	    return 0;
	else
	    return (mu - x)/sigma2;
    }

    public RealInterval gradLogRange(RealInterval xrange) {
	double top = (mu - xrange.beginning)/sigma2;
	double bot = (mu - xrange.end)/sigma2;
	if( bot < top )
	    return new RealInterval( bot, top );
	else
	    return new RealInterval( top, bot );
    }

    public RealInterval range(RealInterval r) {

	double t = r.beginning - mu;
	double a = Math.exp( - t * t / (2 * sigma2) - logmax);
	t = r.end - mu;
	double b = Math.exp( - t * t / (2 * sigma2) - logmax);
	
	if( r.beginning < mu ) {
	    if( r.end < mu )
		return new RealInterval( a, b);
	    else
		return new RealInterval( Math.min(a, b), 1);
	} else
	    return new RealInterval( b, a);
    }
    
    public double getMean() {
	return mu;
    }

    public double getVariance() {
	return sigma2;
    }

//     public double getNormalizingConstant() { // for interval (0,1)
// 	double d = 1/1000;
// 	double x = d/2;
// 	double riemann = 0;
// 	for( int i = 0; i < 999; i++) {
// 	    double t = x - mu;
// 	    riemann += Math.exp( - t * t/(2 * sigma2));
// 	    x += d;
// 	}
// 	return riemann * d;	    
//     }
}
