/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */


import java.awt.*;

/**
 * This class implements a real-valued function defined on a subinterval
 * of the real line.
 * @author L.A. Breyer
 * @version 1.0
 */
abstract public class RealFunction {

	/**
	 * This method returns the value of the function at a point.
	 * @param x the point at which to evaluate the function.
	 * @return the value of the function at the point x.
	 */
    abstract public double valueAt( double x);

	/**
	 * This method returns a RealInterval representing the approximate
	 * range of the function over the domain xRange. The range is computed
	 * by sampling along a grid of size stepSize.
	 * @param xRange a RealInterval specifying the domain of interest
	 * @param stepSize a step size for the grid
	 * @return a RealInterval bounding the minimum and maximum values of
	 * the function.
	 * @see RealInterval
	 */
    public RealInterval getRange(RealInterval xRange, double stepSize) { 
	
	RealInterval yRange = new RealInterval(0,0);
	
	for( double x = xRange.beginning; x < xRange.end; x += stepSize ) {
	    double y = valueAt(x);
	    yRange.beginning = ( yRange.beginning < y ?
				 yRange.beginning : y );
	    yRange.end = ( yRange.end > y ? yRange.end : y );
	}
	return yRange;
    }

	/**
	 * This method displays a polygonal approximation of the graph of
	 * the function over a prespecified domain and range.
	 * @param g a Graphics object for displaying the graph.
	 * @param d a Dimension describing the size of the display window
	 * @param xRange a RealInterval representing the domain
	 * @param yRange a RealInterval representing the range
	 * @see RealInterval
	 */
    public void displayGraph(Graphics g, Dimension d, RealInterval xrange,
			     RealInterval yRange) {
	
	double xStep = (xrange.end - xrange.beginning)/ d.width;
	double yMag = (double)(d.height / (yRange.end - yRange.beginning));
	double yMin = yRange.beginning;

	double x = xrange.beginning;
	int syold = d.height - (int)((valueAt(x)- yMin) * yMag);
	
	for( int sx =0, sy=0; sx < d.width; sx++ ) {
	    x += xStep;
	    sy = d.height - (int)((valueAt(x) - yMin) * yMag);
	    g.drawLine(sx, syold, sx + 1, sy);
	    syold = sy;
	}
    }

	/**
	 * This method displays a polygonal approximation of the graph of
	 * the function over a prespecified domain. The range is computed
	 * automatically.
	 * @param g a Graphics object for displaying the graph.
	 * @param d a Dimension describing the size of the display window
	 * @param xRange a RealInterval representing the domain
	 * @see RealInterval
	 */
    public void displayGraph(Graphics g, Dimension d, RealInterval xRange) {
	
	double xStep = (xRange.end - xRange.beginning)/ d.width;
	displayGraph(g, d, xRange, getRange( xRange, xStep ));
    }
}




