/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;

class TargetAverage extends Canvas {

    int numslots, numpoints, slots[], numAccepted;
    double discretePdf[];
    double magnification = 1.0;
    double normalizingConstant;
    double oldpoint = -1.0;

    double meanSum, theoMean, theoVar;
    Dimension dold;
    Image sbuf = null;
    
    TargetAverage() {
	setBackground(Color.white);
	setForeground(Color.blue);
	dold = new Dimension(0,0);
    }

    public void addPoint(double newpoint) {
	if( (numslots < 1) || (newpoint < 0) || (newpoint > 1) )
	    return;
	
	int place = (int)Math.floor(newpoint * numslots);
	slots[place]++;
	numpoints++;
	if(newpoint != oldpoint) {
	    numAccepted++;
	    oldpoint = newpoint;
	}

	meanSum += newpoint;
    }

    public void reset(int n) {
	numslots = n;
	magnification = 1.0;
	slots = new int[numslots];
 	discretePdf = new double[numslots]; 
	numpoints = 0;
	oldpoint = -1.0;
	numAccepted = 0;
	meanSum = 0;
	theoMean = 0;
	theoVar = 0;
    }

	/**
	 * This method computes the normalizing constant (approximately) for the
	 * pdf, so as to magnify the histogram correctly on screen. Also, a
	 * simple discretization of the pdf is done for estimating the total
	 * variation distance of the histogram to the limiting distribution
	 */
    public void calibrate(Pdf pdf) {
	if( (pdf == null) || (numslots < 1) ) {
	    magnification = 0.0;
	    return;
	}
	
//	Dimension d = getSize();

	double x = (double)1/4000;
	double h = (double)1/2000; //d.width;

	normalizingConstant = pdf.valueAt(0);
	theoMean = 0.0;
	theoVar = 0.0;
	for(int i = 0; i < 1999; i++) { // compute normalizing const and mean
	    double s = pdf.valueAt(x);
	    normalizingConstant += s;
	    theoMean += x * s;
	    theoVar += x * x * s;
	    
	    x += h;
	}
	normalizingConstant *= h;
	theoMean *= h/ normalizingConstant;
	theoVar = (theoVar * h/normalizingConstant) - theoMean * theoMean;
	
	RealInterval dr = pdf.getDensityRange();
	magnification = (normalizingConstant * numslots)/(dr.end - dr.beginning);

 	for(int j = 0; j < numslots; j++ ) { // compute pdf discretization
	    discretePdf[j] = 0;
  	    double a = (double)j/numslots;
  	    double b = (double)(j+1)/numslots;
	    double y = a + (double)1/4000;
	    do {
		discretePdf[j] += pdf.valueAt(y);
		y += h;
	    } while( y <= b );
  	    discretePdf[j] *= h/normalizingConstant;	    
	}
    }

    public double getTheoMean() {
	return theoMean;
    }

    public double getTheoVar() {
	return theoVar;
    }
    
    public void paint(Graphics g) {

	Dimension size = getSize();

	if( !size.equals(dold) ) { // new window size
	    sbuf = createImage(size.width, size.height);
	    dold.width = size.width;
	    dold.height = size.height;
	}

	Graphics screengc = g;
	g = sbuf.getGraphics();

	g.setColor(Color.white);
	g.fillRect(0,0,size.width, size.height);

	if((numpoints > 0) && (numslots > 0)) {
	    g.setColor(Color.blue);
	    double w = (double)(size.width) / numslots;
	    int xoffset = (int)Math.floor(w/2);
	    double h = size.height / 2;
	    
	    double tvtotal = 0.0; double hy =0;
	    for(int i = 0; i < numslots; i++) {
		hy = (double)slots[i]/numpoints;
		tvtotal += Math.abs( hy - discretePdf[i]);
		
		int y = (int)((slots[i] * h * magnification)/numpoints);
		int py = (int)(discretePdf[i] * h * magnification);
		int x = (int)(i * w);
		int wx = (int)Math.floor(((i+1) * w) - x) + 1;
		g.setColor(Color.blue);
		g.fillRect(x, size.height - y, wx, y);
		
		if( py < y ) {
		    g.setColor(Color.cyan);
		    g.fillRect(x, size.height - y, wx, y - py);
		} else {
		    g.setColor(Color.gray);
		    g.fillRect(x, size.height - py, wx, py - y);
		}
//	    g.drawLine(xoffset + x, size.height - y, xoffset + x, size.height);
	    }
	    g.setColor(Color.red);
	    int mean = (int)(meanSum * size.width/numpoints);
	    int tmean = (int)(theoMean * size.width);
	    g.drawLine( mean, 0, mean, size.height);
	    g.setColor(Color.green);
	    g.drawLine( tmean, 0, tmean, size.height);
	    
	    g.setColor(Color.black);
	    int ap = 100 * numAccepted / numpoints;
//	double itv = (int)(100 * tvtotal);
	    g.drawString("Accepted moves : " + ap + "%", 10, 15);
	    g.drawString("Iterations : " + numpoints, 10, 30);
	    StringBuffer sbuf = new StringBuffer();
	    sbuf.append(tvtotal);
	    sbuf.setLength(5);
	    g.drawString("TV to discretized pdf : " +  sbuf , 10, 45);
	    g.drawRect(0,0,size.width-1, size.height-1);
	}
	
	screengc.drawImage(sbuf, 0, 0, null);
    }

    public void update(Graphics g) {
	paint(g);
    }
    
}
