/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class implements a random field suitable for using
 * as a proposal with the Random walk Metropolis sampler, using a uniform
 * proposal distribution.
 * @author L.A. Breyer
 * @version 1.0
 */
final class UniformRandomWalkField extends CouplableField { // does not implement GaussianEstimates

    double w;
    double width;
    double widthon2;

	/**
	 * This constructor accepts a value of
	 * s, the magnification factor for the increment.
	 * @param r the random number generator
	 * @param s half the width of the increment
	 */
    UniformRandomWalkField(Random r, double s) {
	w = r.nextDouble();
	width = 2 * s;
	widthon2 = s;
	w = width * (w - 0.5);
    }

    UniformRandomWalkField respawn(Random r, double s) {
	w = r.nextDouble();
	width = 2 * s;
	w = width * (w - 0.5);
	return this;
    }

    public double valueAt(double x) {
	return x + w;
    }

    public double sigmaAt(double z) {
	return w * w * w / 12;
    }

    public double muAt(double z) {
	return z;
    }

    public RealInterval sigmaRange(RealInterval r) {
	double s = w * w * w / 12;
	return new RealInterval(s, s);
    }

    public RealInterval muRange(RealInterval r) {
	return r;
    }
    
//  	/**
//  	 * note: estimate assumes that r is a subinterval of [0,1]
//  	 */
//      public double bEstimate(RealInterval r, double x, double y) {
//  	double t = y * y - x * x;
//  	if( y > x )
//  	    t -= 2 * r.end * (y - x);
//  	else
//  	    t -= 2 * r.beginning * (y - x);

//  	return Math.exp( - t / (2 * sigma2) );
//      }

//  	/**
//  	 * note: estimate assumes that r is a subinterval of [0,1]. 
//  	 */
//      public double bTildeEstimate(RealInterval r, double x, double y) {
//  	double t = y * y - x * x;
//  	if( y > x )
//  	    t -= 2 * r.beginning * (y - x);
//  	else
//  	    t -= 2 * r.end * (y - x);

//  	return Math.exp( - t / (2 * sigma2) );
//      }
    
    public double newVariableAt(Random r, double x) {
	double w = r.nextDouble();
	return x + width * (w - 0.5);
    }

    public double kernelDensity(double x, double y) {
	if( (y < x - widthon2) || (y > x + widthon2) )
	    return 0;
	else
	    return 1;
    }

    public double maxSigma(double z0, double eps) {
	return w * w * w / 12;
    }

    public double maxMu(double z0, double eps) {
	return eps;
    }

    public double centerMu(double z0) {
	return z0;
    }    

//    public double kernelMean(double x) {
//	return x;
//   }

//    public double kernelVariance(double x) {
//	return sigma2;
//    }

}




